package com.qmth.distributed.print.business.bean.dto.report;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.annotations.ApiModelProperty;
import org.jetbrains.annotations.NotNull;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * @Description: 报告课程考核成绩评价明细结果-考生-目标dto
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/2/26
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class ReportExamStudentTargetDto implements Serializable, Comparable<ReportExamStudentTargetDto> {

    @ApiModelProperty(value = "课程目标id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long targetId;

    @ApiModelProperty(value = "课程目标名称")
    private String targetName;

    @ApiModelProperty(value = "毕业要求")
    private String graduationRequirement;

    @ApiModelProperty(value = "目标整体权重")
    private BigDecimal targetWeight = new BigDecimal(0);

    @ApiModelProperty(value = "目标整体分值")
    private BigDecimal targetScore = new BigDecimal(0);

    @ApiModelProperty(value = "目标达成评价值")
    private BigDecimal evaluationValue = new BigDecimal(0);

    @ApiModelProperty(value = "期末成绩")
    private ReportExamStudentFinalScoreDto finalScore;

    @ApiModelProperty(value = "平时成绩")
    private ReportExamStudentUsualScoreDto usualScore;

    public ReportExamStudentTargetDto() {

    }

    public ReportExamStudentTargetDto(CourseTargetWordDto courseTargetWordDto) {
        this.targetId = courseTargetWordDto.getTargetId();
        this.targetName = courseTargetWordDto.getTargetName();
        this.graduationRequirement = courseTargetWordDto.getGraduationRequirement();
        this.targetWeight = courseTargetWordDto.getTargetWeight();
        this.targetScore = courseTargetWordDto.getTargetScore();
    }

    public ReportExamStudentTargetDto(CourseTargetWordDto courseTargetWordDto, ReportExamStudentFinalScoreDto finalScore, ReportExamStudentUsualScoreDto usualScore) {
        this.targetId = courseTargetWordDto.getTargetId();
        this.targetName = courseTargetWordDto.getTargetName();
        this.graduationRequirement = courseTargetWordDto.getGraduationRequirement();
        this.targetWeight = courseTargetWordDto.getTargetWeight();
        this.targetScore = courseTargetWordDto.getTargetScore();
        this.finalScore = finalScore;
        this.usualScore = usualScore;
    }

    public BigDecimal getTargetScore() {
        return targetScore;
    }

    public void setTargetScore(BigDecimal targetScore) {
        this.targetScore = targetScore;
    }

    public Long getTargetId() {
        return targetId;
    }

    public void setTargetId(Long targetId) {
        this.targetId = targetId;
    }

    public String getTargetName() {
        return targetName;
    }

    public void setTargetName(String targetName) {
        this.targetName = targetName;
    }

    public ReportExamStudentFinalScoreDto getFinalScore() {
        return finalScore;
    }

    public void setFinalScore(ReportExamStudentFinalScoreDto finalScore) {
        this.finalScore = finalScore;
    }

    public ReportExamStudentUsualScoreDto getUsualScore() {
        return usualScore;
    }

    public void setUsualScore(ReportExamStudentUsualScoreDto usualScore) {
        this.usualScore = usualScore;
    }

    public String getGraduationRequirement() {
        return graduationRequirement;
    }

    public void setGraduationRequirement(String graduationRequirement) {
        this.graduationRequirement = graduationRequirement;
    }

    public BigDecimal getTargetWeight() {
        return targetWeight;
    }

    public void setTargetWeight(BigDecimal targetWeight) {
        this.targetWeight = targetWeight;
    }

    public BigDecimal getEvaluationValue() {
        return evaluationValue;
    }

    public void setEvaluationValue(BigDecimal evaluationValue) {
        this.evaluationValue = evaluationValue;
    }

    @Override
    public int compareTo(@NotNull ReportExamStudentTargetDto o) {
        if (o.getTargetId().longValue() < this.getTargetId().longValue()) {
            return 1;
        } else if (o.getTargetId().longValue() > this.getTargetId().longValue()) {
            return -1;
        } else {
            return 0;
        }
    }
}
